@@ -2,6 +2,7 @@ import contextlib
 import glob
 import hashlib
 import os
+import re
 import shutil
 import stat
 import subprocess
@@ -11,10 +12,10 @@ import zipfile
 from base64 import urlsafe_b64encode
 from io import StringIO
 from pathlib import Path
-from typing import List, Tuple, Union
+from typing import Any, Generator, List, Mapping, Optional, TextIO, Tuple, Union
 
 from pdm.pep517 import __version__
-from pdm.pep517._vendor.packaging.markers import default_environment
+from pdm.pep517._vendor.packaging import tags
 from pdm.pep517._vendor.packaging.specifiers import SpecifierSet
 from pdm.pep517.base import Builder, BuildError
 from pdm.pep517.utils import get_abi_tag, get_platform, safe_version, to_filename
@@ -29,13 +30,37 @@ Tag: {tag}
     % __version__
 )
 
+PY_LIMITED_API_PATTERN = r"cp3\d"
+
 
 class WheelBuilder(Builder):
-    def __init__(self, location: Union[str, Path]) -> None:
-        super().__init__(location)
+    def __init__(
+        self,
+        location: Union[str, Path],
+        config_settings: Optional[Mapping[str, Any]] = None,
+    ) -> None:
+        super().__init__(location, config_settings)
         self._records = []  # type: List[Tuple[str, str, str]]
+        self._parse_config_settings()
 
-    def build(self, build_dir: str, **kwargs) -> str:
+    def _parse_config_settings(self) -> None:
+        self.python_tag = None
+        self.py_limited_api = None
+        self.plat_name = None
+        if not self.config_settings:
+            return
+        if "--python-tag" in self.config_settings:
+            self.python_tag = self.config_settings["--python-tag"]
+        if "--py-limited-api" in self.config_settings:
+            self.py_limited_api = self.config_settings["--py-limited-api"]
+            if not re.match(PY_LIMITED_API_PATTERN, self.py_limited_api):
+                raise ValueError(
+                    "py-limited-api must match '%s'" % PY_LIMITED_API_PATTERN
+                )
+        if "--plat-name" in self.config_settings:
+            self.plat_name = self.config_settings["--plat-name"]
+
+    def build(self, build_dir: str, **kwargs: Any) -> str:
         if not os.path.exists(build_dir):
             os.makedirs(build_dir, exist_ok=True)
 
@@ -65,40 +90,34 @@ class WheelBuilder(Builder):
 
     @property
     def tag(self) -> str:
+        platform = self.plat_name
+        impl = self.python_tag
         if self.meta.build:
-            info = default_environment()
-            platform = get_platform()
-            implementation = info["implementation_name"]
-            impl_name = (
-                "cp"
-                if implementation.startswith("cp")
-                else "jp"
-                if implementation.startswith("jp")
-                else "ip"
-                if implementation.startswith("ir")
-                else "pp"
-                if implementation.startswith("pypy")
-                else "unknown"
-            )
-            impl_ver = (
-                info["python_full_version"].replace(".", "")
-                if impl_name == "pp"
-                else info["python_version"].replace(".", "")
-            )
-            impl = impl_name + impl_ver
-            abi_tag = get_abi_tag()
-            tag = (impl, abi_tag, platform)
-        else:
-            platform = "any"
-            if self.meta.requires_python and SpecifierSet(
-                self.meta.requires_python
-            ).contains("2.7"):
-                impl = "py2.py3"
+            if not platform:
+                platform = get_platform(self.location / "build")
+            if not impl:
+                impl = tags.interpreter_name() + tags.interpreter_version()
+            if self.py_limited_api and impl.startswith("cp3"):
+                impl = self.py_limited_api
+                abi_tag = "abi3"
             else:
-                impl = "py3"
-
-            tag = (impl, "none", platform)
+                abi_tag = str(get_abi_tag()).lower()
+        else:
+            if not platform:
+                platform = "any"
+            abi_tag = "none"
+            if not impl:
+                if self.meta.requires_python and SpecifierSet(
+                    self.meta.requires_python
+                ).contains("2.7"):
+                    impl = "py2.py3"
+                else:
+                    impl = "py3"
 
+        platform = platform.lower().replace("-", "_").replace(".", "_")
+        tag = (impl, abi_tag, platform)
+        supported_tags = [(t.interpreter, t.abi, platform) for t in tags.sys_tags()]
+        assert tag in supported_tags, f"would build wheel with unsupported tag {tag}"
         return "-".join(tag)
 
     @property
@@ -107,12 +126,12 @@ class WheelBuilder(Builder):
         version = to_filename(safe_version(self.meta.version))
         return f"{name}-{version}.dist-info"
 
-    def _write_record(self, fp):
+    def _write_record(self, fp: TextIO) -> None:
         for row in self._records:
             fp.write("{},sha256={},{}\n".format(*row))
         fp.write(self.dist_info_name + "/RECORD,,\n")
 
-    def _write_metadata(self, wheel):
+    def _write_metadata(self, wheel: zipfile.ZipFile) -> None:
         dist_info = self.dist_info_name
         if self.meta.entry_points:
             with self._write_to_zip(wheel, dist_info + "/entry_points.txt") as f:
@@ -133,7 +152,9 @@ class WheelBuilder(Builder):
             self._write_record(f)
 
     @contextlib.contextmanager
-    def _write_to_zip(self, wheel, rel_path):
+    def _write_to_zip(
+        self, wheel: zipfile.ZipFile, rel_path: str
+    ) -> Generator[StringIO, None, None]:
         sio = StringIO()
         yield sio
 
@@ -150,13 +171,13 @@ class WheelBuilder(Builder):
         print(f" - Adding {rel_path}")
         self._records.append((rel_path, hash_digest, str(len(b))))
 
-    def _build(self, wheel):
+    def _build(self, wheel: zipfile.ZipFile) -> None:
         if not self.meta.build:
             return
         setup_py = self.ensure_setup_py()
         build_args = [
             sys.executable,
-            setup_py,
+            str(setup_py),
             "build",
             "-b",
             str(self.location / "build"),
@@ -169,17 +190,26 @@ class WheelBuilder(Builder):
         lib_dir = next(build_dir.glob("lib.*"), None)
         if not lib_dir:
             return
+
+        _, excludes = self._get_include_and_exclude_paths(for_sdist=False)
         for pkg in lib_dir.glob("**/*"):
             if pkg.is_dir():
                 continue
 
-            rel_path = pkg.relative_to(lib_dir).as_posix()
+            whl_path = rel_path = pkg.relative_to(lib_dir).as_posix()
+            if self.meta.package_dir:
+                # act like being in the package_dir
+                rel_path = Path(self.meta.package_dir) / rel_path
 
-            if rel_path in wheel.namelist():
+            if self._is_excluded(rel_path, excludes):
                 continue
-            self._add_file(wheel, pkg, rel_path)
 
-    def _copy_module(self, wheel):
+            if whl_path in wheel.namelist():
+                continue
+
+            self._add_file(wheel, pkg, whl_path)
+
+    def _copy_module(self, wheel: zipfile.ZipFile) -> None:
         for path in self.find_files_to_add():
             rel_path = None
             if self.meta.package_dir:
@@ -189,7 +219,9 @@ class WheelBuilder(Builder):
                     pass
             self._add_file(wheel, str(path), rel_path)
 
-    def _add_file(self, wheel, full_path, rel_path=None):
+    def _add_file(
+        self, wheel: zipfile.ZipFile, full_path: str, rel_path: Optional[str] = None
+    ) -> None:
         if not rel_path:
             rel_path = full_path
         if os.sep != "/":
@@ -220,10 +252,10 @@ class WheelBuilder(Builder):
 
         self._records.append((rel_path, hash_digest, str(size)))
 
-    def _write_metadata_file(self, fp):
+    def _write_metadata_file(self, fp: TextIO) -> None:
         fp.write(self.format_pkginfo())
 
-    def _write_wheel_file(self, fp):
+    def _write_wheel_file(self, fp: TextIO) -> None:
         fp.write(
             WHEEL_FILE_FORMAT.format(
                 pure_lib=self.meta.build is None,
@@ -231,7 +263,7 @@ class WheelBuilder(Builder):
             )
         )
 
-    def _write_entry_points(self, fp):
+    def _write_entry_points(self, fp: TextIO) -> None:
         entry_points = self.meta.entry_points
         for group_name in sorted(entry_points):
             fp.write("[{}]\n".format(group_name))
